PBI_AUTHORITY =  "https://login.microsoftonline.com/common"
PBI_RESOURCE = "https://analysis.windows.net/powerbi/api"

#API strings
API_BASE = "https://api.powerbi.com/v1.0/myorg/"
API_BASE_GROUP = API_BASE + "groups/{wks_id}/"
